"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ReferenceMap = void 0;
class ReferenceMap {
    constructor() {
        this._itemMap = new Map();
        this._namedFulfillers = [];
    }
    static create() {
        return new ReferenceMap();
    }
    fulfillItemWith(fulfiller) {
        this._namedFulfillers.push(fulfiller);
        return this;
    }
    ensureItemOrDefault(key, defaultItem) {
        if (!this._itemMap.has(key)) {
            return defaultItem;
        }
        return this._itemMap.get(key);
    }
    ensureItemOrUndefined(key) {
        if (!this._itemMap.has(key)) {
            return undefined;
        }
        return this._itemMap.get(key);
    }
    ensureItemOrNull(key) {
        if (!this._itemMap.has(key)) {
            return null;
        }
        return this._itemMap.get(key);
    }
    reset(key) {
        this._itemMap.delete(key);
        return this;
    }
    getItemOrDefault(key, defaultItem) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.fulfillItem(key);
            if (!this._itemMap.has(key)) {
                return defaultItem;
            }
            return this._itemMap.get(key);
        });
    }
    getItemOrUndefined(key) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.fulfillItem(key);
            if (!this._itemMap.has(key)) {
                return undefined;
            }
            return this._itemMap.get(key);
        });
    }
    getItemOrNull(key) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.fulfillItem(key);
            if (!this._itemMap.has(key)) {
                return null;
            }
            return this._itemMap.get(key);
        });
    }
    refreshItem(key) {
        return __awaiter(this, void 0, void 0, function* () {
            this.reset(key);
            return yield this.fulfillItem(key);
        });
    }
    batchWith(fulfiller) {
        return __awaiter(this, void 0, void 0, function* () {
            const shouldUse = yield fulfiller.shouldFulfillWith();
            if (!shouldUse) {
                return false;
            }
            const result = yield fulfiller.execute();
            for (const each of result) {
                this._itemMap.set(each.key, each.value);
            }
            return true;
        });
    }
    fulfillItem(key) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this._itemMap.has(key)) {
                return false;
            }
            for (const fulfiller of this._namedFulfillers) {
                const shouldUse = yield fulfiller.shouldFulfillWith(key);
                if (shouldUse) {
                    const value = yield fulfiller.execute(key);
                    this._itemMap.set(key, value);
                    return true;
                }
            }
            return false;
        });
    }
}
exports.ReferenceMap = ReferenceMap;
